from django.core.management import call_command
from django.urls import reverse
from mapbox_baselayer.models import MapBaseLayer
from rest_framework import status
from rest_framework.test import APITestCase


class MapboxBaseLayerViewsSetTestCase(APITestCase):
    @classmethod
    def setUpTestData(cls):
        call_command('install_osm_baselayer')

    def test_create(self):
        data = {
            "name": "test",
            "base_layer_type": "mapbox",
            "map_box_url": "mapbox://test/",
            "order": 0,
            "min_zoom": 0,
            "max_zoom": 20,
            "tile_size": 256
        }
        response = self.client.post(reverse('baselayer-list'), data=data)
        data = response.json()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED, data)

        self.assertTrue(data['id'])
        self.assertEqual(data['tiles'], [])

    def test_full_update(self):
        data = {
            "name": "test2",
            "base_layer_type": "mapbox",
            "map_box_url": "mapbox://test/",
            "order": 0,
            "min_zoom": 0,
            "max_zoom": 20,
            "tile_size": 256
        }
        pk = MapBaseLayer.objects.first().pk
        response = self.client.patch(reverse('baselayer-detail', args=(pk, )), data=data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertEqual(data['name'], "test2")



from django.conf import settings
from django.urls import path, include
from rest_framework.routers import SimpleRouter
from terra_settings.views import SettingsView

router = SimpleRouter()

if 'mapbox_baselayer' in settings.INSTALLED_APPS:
    from terra_settings.base_layers.views import BaseLayerViewSet
    # enable mapbox_baselayer related endpoints if module enabled in backend
    router.register('baselayer', BaseLayerViewSet, basename='baselayer')

urlpatterns = [
    # public api view
    path('settings/', SettingsView.as_view(), name='settings'),
    # router urls
    path('', include(router.urls)),
]



from django.conf import settings
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from terra_settings.settings import TERRA_APPLIANCE_SETTINGS


class SettingsView(APIView):
    """ This is public endpoint used to init terralego apps """
    permission_classes = (AllowAny, )

    def get(self, request):
        terra_settings = {
            # for the moment, language is fixed and defined by backend instance
            'language': settings.LANGUAGE_CODE.lower(),
        }

        if 'mapbox_baselayer' in settings.INSTALLED_APPS:
            from mapbox_baselayer.models import MapBaseLayer
            from terra_settings.base_layers.serializers import PublicMapBaseLayerSerializer

            terra_settings['base_layers'] = PublicMapBaseLayerSerializer(MapBaseLayer.objects.all(), many=True).data

        # always override terra_settings data with TERRA_APPLIANCE_SETTINGS content
        terra_settings.update(TERRA_APPLIANCE_SETTINGS)

        return Response(terra_settings)



from django.core.management import call_command
from django.urls import reverse
from mapbox_baselayer.models import MapBaseLayer
from rest_framework import status
from rest_framework.test import APITestCase


class MapBaseLayerViewsSetTesCase(APITestCase):
    @classmethod
    def setUpTestData(cls):
        call_command('install_osm_baselayer')
        cls.mapbox = MapBaseLayer.objects.create(name="mapbox", base_layer_type="mapbox", map_box_url="mapbox://test/")

    def test_list(self):
        response = self.client.get(reverse('baselayer-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertTrue(len(data['results']) > 0)

    def test_detail_raster(self):
        pk = MapBaseLayer.objects.get(slug='osm').pk
        response = self.client.get(reverse('baselayer-detail', args=(pk, )))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertEqual(data['name'], 'OSM')

    def test_detail_mapbox(self):
        response = self.client.get(reverse('baselayer-detail', args=(self.mapbox.pk, )))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertEqual(data['name'], 'mapbox')

    def test_tilejson_url_in_list(self):
        response = self.client.get(reverse('baselayer-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertTrue(len(data['results']) > 1)

        # We expect the result to be at index 1 because of the ordering
        osm_layer = MapBaseLayer.objects.get(slug='osm')
        self.assertEqual(data['results'][1]['tilejson_url'], f'http://testserver/baselayer/{osm_layer.id}/tilejson/')

    def test_tilejson_in_list(self):
        pk = MapBaseLayer.objects.get(slug='osm').pk
        response = self.client.get(reverse('baselayer-detail', args=(pk, )))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        tilejson_url = data['tilejson_url']
        response = self.client.get(tilejson_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertTrue(response.json())

    def test_create(self):
        data = {
            "tiles": [
                "//test.org",
            ],
            "name": "test",
            "order": 0,
            "base_layer_type": "raster",
            "min_zoom": 0,
            "max_zoom": 20,
            "tile_size": 256
        }
        response = self.client.post(reverse('baselayer-list'), data=data)
        data = response.json()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED, data)

        self.assertTrue(data['id'])
        self.assertEqual(data['tiles'][0], "//test.org")

    def test_partial_update(self):
        data = {
            "tiles": [
                "//test2.org",
            ],
        }
        pk = MapBaseLayer.objects.first().pk
        response = self.client.patch(reverse('baselayer-detail', args=(pk, )), data=data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertEqual(data['tiles'][0], "//test2.org")

    def test_full_update(self):
        data = {
            "tiles": [
                "//test3.org",
            ],
            "name": "test",
            "order": 0,
            "base_layer_type": "raster",
            "min_zoom": 0,
            "max_zoom": 20,
            "tile_size": 256
        }
        pk = MapBaseLayer.objects.first().pk
        response = self.client.patch(reverse('baselayer-detail', args=(pk, )), data=data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertEqual(data['tiles'][0], "//test3.org")

    def test_delete(self):
        pk = MapBaseLayer.objects.first().pk
        response = self.client.delete(reverse('baselayer-detail', args=(pk, )))
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)



from django.conf import settings

TERRA_APPLIANCE_SETTINGS = getattr(settings, 'TERRA_APPLIANCE_SETTINGS', {})
FRONT_URL = getattr(settings, 'FRONT_URL', 'http://localhost:3000')
HOSTNAME = getattr(settings, 'HOSTNAME', 'http://localhost:8000')

MEDIA_ACCEL_REDIRECT = getattr(settings, 'MEDIA_ACCEL_REDIRECT', False)



from django.apps import AppConfig
from django.conf import settings
from django.utils.translation import gettext_lazy as _

if "terra_accounts" in settings.INSTALLED_APPS:

    from terra_accounts.permissions_mixins import PermissionRegistrationMixin

    class TerraSettingsConfig(PermissionRegistrationMixin, AppConfig):
        name = "terra_settings"

        permissions = (
            ("BaseLayer", "can_manage_baselayers", _("Can manage base map layers")),
        )


else:

    class TerraSettingsConfig(AppConfig):
        name = "terra_settings"



from django.utils.translation import gettext as _
from mapbox_baselayer.models import MapBaseLayer, BaseLayerTile
from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.reverse import reverse


class BaseLayerTileSerializer(serializers.ModelSerializer):
    class Meta:
        model = BaseLayerTile
        fields = ('url', )

    def to_representation(self, instance):
        return instance.url

    def to_internal_value(self, data):
        return data


class MapBaseLayerSerializer(serializers.ModelSerializer):
    tiles = BaseLayerTileSerializer(many=True, required=False)
    tilejson_url = serializers.SerializerMethodField()

    def get_tilejson_url(self, instance):
        """
        Provide generated tilejson or mapbox url along base layer type
        """
        if instance.base_layer_type != 'mapbox':
            return reverse('baselayer-tilejson', args=(instance.pk, ),
                           request=self.context.get('request'))
        else:
            return instance.map_box_url

    def validate(self, attrs):
        if attrs.get('base_layer_type') == 'mapbox':
            if attrs.get('tiles'):
                raise ValidationError(_("'tiles' should not be filled for mapbox base layers."))
            if not attrs.get('map_box_url'):
                raise ValidationError(_("'map_box_url' should be filled for mapbox base layers."))
        else:
            if not attrs.get('tiles'):
                raise ValidationError(_("'tiles' should be filled for raster and vector base layers."))
            if attrs.get('map_box_url'):
                raise ValidationError(_("'map_box_url' should not be filled for raster and vector base layers."))
        return attrs

    def create(self, validated_data):
        """ handle sub tile definition """
        tiles = validated_data.pop('tiles', [])
        instance = MapBaseLayer.objects.create(**validated_data)
        for tile in tiles:
            BaseLayerTile.objects.create(base_layer=instance, url=tile)
        return instance

    def update(self, instance, validated_data):
        """ handle sub tile definition """
        tiles = validated_data.pop('tiles', [])
        for key, value in validated_data.items():
            # update data for each instance validated value
            setattr(instance, key, value)
        if tiles:
            # delete tiles not in putted data
            instance.tiles.exclude(url__in=[tile for tile in tiles]).delete()
            tile_instances = [
                BaseLayerTile.objects.get_or_create(url=tile,
                                                    base_layer=instance)[0] for tile in tiles
            ]
            instance.tiles.set(tile_instances)
        instance.save()
        return instance

    class Meta:
        model = MapBaseLayer
        fields = (
            'id', 'tiles', 'name', 'order', 'slug',
            'base_layer_type', 'map_box_url', 'sprite',
            'glyphs', 'min_zoom', 'max_zoom',
            'tile_size', 'attribution', 'tiles', 'tilejson_url'
        )


class PublicMapBaseLayerSerializer(serializers.ModelSerializer):
    """ Serializer used to provide quick select list in default public settings API endpoint """
    tilejson_url = serializers.SerializerMethodField()

    def get_tilejson_url(self, instance):
        """
        Provide generated tilejson or mapbox url along base layer type
        """
        if instance.base_layer_type != 'mapbox':
            return reverse('baselayer-tilejson', args=(instance.pk, ))
        else:
            return instance.map_box_url

    class Meta:
        model = MapBaseLayer
        fields = (
            'id', 'order', 'name', 'slug', 'tilejson_url'
        )



from mapbox_baselayer.models import MapBaseLayer
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from terra_settings.base_layers.serializers import MapBaseLayerSerializer


class BaseLayerViewSet(viewsets.ModelViewSet):
    serializer_class = MapBaseLayerSerializer
    queryset = MapBaseLayer.objects.all()

    @action(detail=True)
    def tilejson(self, request, *args, **kwargs):
        """ Full tilejson """
        base_layer = self.get_object()
        return Response(base_layer.tilejson)



import os

from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase
from django.test.client import RequestFactory
from rest_framework import status

from terra_settings.helpers.responses import get_media_response


class MediaResponseTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_get_media_response_with_path(self):
        request = self.factory.get('fake/path')

        # Creating a real file
        # In this case, we test if get_media_response return content
        # from a file and so will use open()
        tmp_name = '/tmp/test.txt'
        with open(tmp_name, 'wb') as tmp_file:
            tmp_file.write(b"ceci n'est pas une pipe")

        response = get_media_response(
            request,
            {'path': tmp_name, 'url': None}  # url none, no accel-redirect
        )

        # deleting the file since we don't need it anymore
        os.remove(tmp_name)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIsInstance(response.content, bytes)
        self.assertEqual(response.content, b"ceci n'est pas une pipe")

    def test_get_media_response_with_file_object(self):
        request = self.factory.get('fake/path')

        tmp_file = SimpleUploadedFile(name='/tmp/file.txt',
                                      content=b'creativity takes courage')

        # Adding fake url, we don't care, we are not testing accel-redirect
        tmp_file.url = None

        response = get_media_response(request, tmp_file)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIsInstance(response.content, bytes)
        self.assertEqual(response.content, b'creativity takes courage')



from django.template import Context, Template
from django.test import TestCase

from terra_settings import settings


class TemplatesTagsTestCase(TestCase):
    def test_front_url(self):
        context = Context()
        template_to_render = Template(
            '{% load settings_tags %}'
            '{% front_url %}'
        )

        rendered = template_to_render.render(context)

        self.assertEqual(rendered, settings.FRONT_URL)
